// MemTrack.h
// Copyright (C) 2000, Jean Louis Gareau
// All rights reserved.
//
// See MemTrack.cpp for a description.

#pragma once

#include <afxtempl.h>

// MEMTRACK_DLL is and must be defined for this project's settings only.

#ifdef MEMTRACK_DLL
#define MEMTRACK_EXT_CLASS __declspec(dllexport)
#define MEMTRACK_EXT_DATA  __declspec(dllexport)
#else
#define MEMTRACK_EXT_CLASS __declspec(dllimport)
#define MEMTRACK_EXT_DATA  __declspec(dllimport)
#endif

/////////////////////////////////////////////////////////////////////////////
// SMEMALLOC - Memory allocation description structure. It describes a
// statement that allocated memory.

struct SMEMALLOC
{
   const char* pszFilename;            // Filename
   int         nLine;                  // Line number
   size_t      nSize;                  // Block size
};

typedef SMEMALLOC *  LPMEMALLOC;

// Build a SMEMALLOC map type, where each item associates an address 
// returned by malloc() to a description of the statement allocating memory.

typedef CTypedPtrMap<CMapPtrToPtr, LPVOID, LPMEMALLOC> CAddrMap;

/////////////////////////////////////////////////////////////////////////////
// CMemTrack - Exported memory check class.

class MEMTRACK_EXT_CLASS CMemTrack: public CObject
{
   CAddrMap    m_AddrMap;              // Internal map
   BOOL        m_bSilent;              // Silent mode on/off

public:

   CMemTrack();

   // Allocation and deallocation tracking methods.
   LPVOID      Allocate(size_t nSize, char* lpszFileName, int nLine);
   void        Deallocate(void* pData);

   // Dump methods.
   void        SetSilentMode(BOOL bMode);
   void        DumpMemoryLeaks();      // Automatically called.
   static void Dump();                 // Can be manually called.

   // Cleanup method.
   void        Cleanup();
};

inline void CMemTrack::SetSilentMode(BOOL bMode)
{
   m_bSilent = bMode;
}

// There is one instance of CMemTrack in the DLL.

extern MEMTRACK_EXT_DATA CMemTrack  afxMemTrack;

/////////////////////////////////////////////////////////////////////////////
// The following must be included in every project but MemTrack itself.

#ifndef __MEMTRACK__

#ifdef _DEBUG
#pragma comment(lib, "MemTrackd.lib")
#else
#pragma comment(lib, "MemTrack.lib")
#endif

// The inlined new operator (used by applications).

inline void* AFX_CDECL operator new(size_t nSize, char* pszFileName, int nLine)
{
   return afxMemTrack.Allocate(nSize, pszFileName, nLine);
}

// The inlined delete operator (used by applications).

inline void AFX_CDECL operator delete(void * pData)
{
   afxMemTrack.Deallocate(pData);
}

// Make sure that the global new and delete operators are being used.
// Also take advantage of the "#define new DEBUG_NEW" (in debug mode 
// only) generated by AppWizard in every .cpp file.

#ifdef   _DEBUG
#undef   DEBUG_NEW
#define  DEBUG_NEW   ::new(THIS_FILE, __LINE__)
#else
#define  new         ::new(THIS_FILE, __LINE__)
#endif

#define  delete      ::delete

#endif   // __MEMTRACK__
